@@ -22,8 +22,12 @@
 
 DirtyAllocator::DirtyAllocator(
     const int dim, const int tabSize
-): tabSize(tabSize), cellSize( sizeof(double) + sizeof(int) * dim )
+): tabSize(tabSize)
 {
+    cellSize        = sizeof(double) + sizeof(int) * dim;
+    // Fix memory alignment problems for SPARC
+    if(cellSize % sizeof(double) != 0)
+    	cellSize += sizeof(double) - cellSize % sizeof(double);
     currentTab      = malloc( cellSize * tabSize );
     currentConf     = currentTab;
     endOfTablePtr = reinterpret_cast<char*>(currentTab) + cellSize*tabSize;
